<?php
namespace BootstrapDefault;

class Config {

    const CONF_PARAM = 'bootstrapdefault';
    const CONF_VERSION = 2;

    const TYPE_BOOL = 'bool';
    const TYPE_STRING = 'string';

    const KEY_VERSION = 'conf_version';

    const KEY_BOOTSTRAP_THEME = 'bootstrap_theme';

    const KEY_SOCIAL_ENABLED = 'social_enabled';
    const KEY_SOCIAL_TWITTER = 'social_twitter';
    const KEY_SOCIAL_FACEBOOK = 'social_facebook';
    const KEY_SOCIAL_GOOGLE_PLUS = 'social_google_plus';

    const KEY_COMMENTS_TYPE = 'comments_type';
    const KEY_COMMENTS_DISQUS_SHORTNAME = 'comments_disqus_shortname';

    const KEY_TAG_CLOUD_TYPE = 'tag_cloud_type';

    private $defaults = array(
        self::KEY_BOOTSTRAP_THEME => 'default',
        self::KEY_SOCIAL_ENABLED => true,
        self::KEY_SOCIAL_TWITTER => true,
        self::KEY_SOCIAL_FACEBOOK => true,
        self::KEY_SOCIAL_GOOGLE_PLUS => true,
        self::KEY_COMMENTS_TYPE => 'piwigo',
        self::KEY_COMMENTS_DISQUS_SHORTNAME => null,
        self::KEY_TAG_CLOUD_TYPE => 'basic',
    );

    private $types = array(
        self::KEY_BOOTSTRAP_THEME => self::TYPE_STRING,
        self::KEY_SOCIAL_ENABLED => self::TYPE_BOOL,
        self::KEY_SOCIAL_TWITTER => self::TYPE_BOOL,
        self::KEY_SOCIAL_FACEBOOK => self::TYPE_BOOL,
        self::KEY_SOCIAL_GOOGLE_PLUS => self::TYPE_BOOL,
        self::KEY_COMMENTS_TYPE => self::TYPE_STRING,
        self::KEY_COMMENTS_DISQUS_SHORTNAME => self::TYPE_STRING,
        self::KEY_TAG_CLOUD_TYPE => self::TYPE_STRING,
    );

    private $config = array();

    public function __construct() {
        global $conf;

        // Create initial config if necessary
        if (!isset($conf[self::CONF_PARAM])) {
            $this->createDefaultConfig();
            return;
        }

        // Load and JSON decode the config
        $loaded = json_decode($conf[self::CONF_PARAM], true);

        // Check for current version
        if (isset($loaded[self::KEY_VERSION]) && $loaded[self::KEY_VERSION] == self::CONF_VERSION) {
            $this->config = $loaded;
            return;
        }

        // Invalid or old config, recreate
        $this->createDefaultConfig();
        if (is_array($loaded)) {
            $this->populateConfig($loaded);
        }
        $this->save();
    }

    public function __set($key, $value) {
        if (array_key_exists($key, $this->defaults)) {
            $this->config[$key] = $value;
        }
    }

    public function __get($key) {
        if (array_key_exists($key, $this->defaults)) {
            return $this->config[$key];
        } else {
            return null;
        }
    }

    public function fromPost(array $post) {
        foreach ($this->defaults as $key => $value) {
            switch ($this->types[$key]) {
                case self::TYPE_STRING:
                    $this->config[$key] = isset($post[$key]) ? $post[$key] : null;
                    break;
                case self::TYPE_BOOL:
                    $this->config[$key] = isset($post[$key]);
                    break;
            }
        }
    }

    public function save() {
        conf_update_param(self::CONF_PARAM, json_encode($this->config));
    }

    private function createDefaultConfig() {
        $this->config = $this->defaults;
        $this->config[self::KEY_VERSION] = self::CONF_VERSION;
    }

    private function populateConfig(array $config) {
        foreach ($this->defaults as $key => $value) {
            if (isset($config[$key])) {
                $this->config[$key] = $config[$key];
            }
        }
    }

}